@@ -20,6 +20,7 @@ functions.
 ###############################################################################
 package XAO::Utils;
 use strict;
+use Encode;
 use XAO::Errors qw(XAO::Utils);
 
 ##
@@ -33,13 +34,14 @@ sub dprint (@);
 sub eprint (@);
 sub t2ht ($);
 sub t2hf ($);
-sub t2hq ($);
+sub t2hq ($;$);
+sub t2hj ($);
 sub get_args (@);
 sub merge_refs (@);
 sub fround ($$);
 
 use vars qw($VERSION);
-$VERSION=(0+sprintf('%u.%03u',(q$Id: Utils.pm,v 2.1 2005/01/13 22:34:34 am Exp $ =~ /\s(\d+)\.(\d+)\s/))) || die "Bad VERSION";
+$VERSION='2.7';
 
 ###############################################################################
 # Export control
@@ -51,7 +53,7 @@ require Exporter;
     all => \@EXPORT_OK,
     args => [qw(get_args merge_refs)],
     debug => [qw(dprint eprint)],
-    html => [qw(t2ht t2hq t2hf)],
+    html => [qw(t2ht t2hq t2hf t2hj)],
     keys => [qw(generate_key repair_key)],
     math => [qw(fround)],
     none => [],
@@ -89,9 +91,6 @@ Here is the list of functions available:
 Generating new 8-characters random ID. Not guaranteed to be unique,
 must be checked against existing database.
 
-You can pass additional argument to add some more randomness, but it is
-not required and is kept for compatibility.
-
 Generated ID is relativelly suitable for humans - it does not contain
 some letters and digits that could be easily misunderstood in writing:
 
@@ -115,27 +114,39 @@ Is similar to U.
 
 =back
 
-Examples of generated IDs are E5TUVX82, ZK845LP6 and so on. Id would
-never start from digit!
+Examples of generated IDs are E5TUVX82, ZK845LP6 and so on.
+
+The generated ID will never start with a digit!
+
+The default generated key length is 8. This can be changed by supplying
+an optional argument -- generate_key(20) for example.
 
 =cut
 
-sub generate_key (;$) {
-    #                        1    1    2    2    3
-    #              0----5----0----5----0----5----0-
-    my $symbols = "2345689ABCDEFGHIJKLMNOPQRSTUWXYZ";
-    my $key='';
+my $generate_key_alpha;
+my $generate_key_alnum;
+my $generate_key_alpha_len;
+my $generate_key_alnum_len;
 
-    while(!$key || $key=~/^[0-9]+$/) {
-        my $rval=pack("SSC",rand(0x10000),rand(0x10000),
-                            rand(0x100)^unpack("%8C*",$_[0] || "foo"));
-        for(my $i=0; $i!=8; $i++) {
-            my $v=vec($rval,$i+2,4) + vec($rval,$i,1)*16;
-            $key.=substr($symbols,$v,1);
-        }
+sub generate_key (;$) {
+    my $length=$_[0] || 8;
+
+    if(!$generate_key_alpha) {
+        #                              1    1    2    2    3
+        #                    0----5----0----5----0----5----0-
+        $generate_key_alpha=        'ABCDEFGHIJKLMNOPQRSTUWXYZ';
+        $generate_key_alnum='2345689'.$generate_key_alpha;
+        $generate_key_alpha_len=length($generate_key_alpha);
+        $generate_key_alnum_len=length($generate_key_alnum);
     }
 
-    $key;
+    my $key=substr($generate_key_alpha,rand($generate_key_alpha_len),1);
+
+    for(my $i=1; $i!=$length; $i++) {
+        $key.=substr($generate_key_alnum,rand($generate_key_alnum_len),1);
+    }
+
+    return $key;
 }
 
 ###############################################################################
@@ -191,6 +202,17 @@ use vars qw($debug_flag $logprint_handler);
 
 ###############################################################################
 
+sub logprint ($) {
+    if($logprint_handler) {
+        &{$logprint_handler}($_[0]);
+    }
+    else {
+        print STDERR $_[0]."\n";
+    }
+}
+
+###############################################################################
+
 =item dprint (@)
 
 Prints all arguments just like normal "print" does but 1) it prints
@@ -215,7 +237,7 @@ debugging for all sites at once.
 
 sub dprint (@) {
     return unless $debug_flag;
-    my $str=join("",map { defined($_) ? $_ : "<UNDEF>" } @_);
+    my $str=join('',map { defined($_) ? $_ : '<UNDEF>' } @_);
     chomp $str;
     logprint($str);
 }
@@ -231,20 +253,9 @@ reporting minor problems to the server log.
 =cut
 
 sub eprint (@) {
-    my $str=join("",map { defined($_) ? $_ : "<UNDEF>" } @_);
+    my $str=join('',map { defined($_) ? $_ : '<UNDEF>' } @_);
     chomp $str;
-    logprint("*ERROR: ",$str);
-}
-
-###############################################################################
-
-sub logprint (@) {
-    if($logprint_handler) {
-        &{$logprint_handler}(join('',@_));
-    }
-    else {
-        print STDERR @_,"\n";
-    }
+    logprint('*ERROR: '.$str);
 }
 
 ###############################################################################
@@ -338,21 +349,21 @@ XAO::Web object arguments as well.
 
  " ->> &quot;
 
-All symbols from 0x0 to 0x1f and from 0x80 to 0x9f are substituted with
-their codes in &#NNN; format.
+All symbols from 0x0 to 0x1f are substituted with their codes in &#NNN;
+format.
 
 =cut
 
 sub t2hf ($) {
     my $text=t2ht($_[0]);
     $text=~s/"/&quot;/sg;
-    $text=~s/([\x00-\x1f\x80-\x9f<>])/'&#'.ord($1).';'/sge;
+    $text=~s/([\x00-\x1f<>])/'&#'.ord($1).';'/sge;
     $text;
 }
 
 ###############################################################################
 
-=item t2hq ($)
+=item t2hq ($;$)
 
 Escapes text to be be included into URL parameters.
 
@@ -361,12 +372,21 @@ symbols from [&?<>"=%#+] are substituted to %XX hexadecimal codes
 interpreted by all standard CGI tools. The same conversion may be used
 for URLs themselves.
 
+Unicode is encoded into UTF-8 (unless a different encoding is specified
+in the second argument).
+
 =cut
 
-sub t2hq ($) {
-    my $text=shift;
-    $text=~s/([\x00-\x20\x80-\xff\&\?<>;"=%#+])/"%".unpack("H2",$1)/sge;
-    $text;
+sub t2hq ($;$) {
+    my ($text,$encoding)=@_;
+
+    my $bytes=Encode::is_utf8($text)
+                ? Encode::encode($encoding || 'utf8',$text)
+                : $text;
+
+    $bytes=~s/([^[:ascii:]]|[\x00-\x20\&\?<>;"=%#\+])/"%".unpack("H2",$1)/sge;
+
+    return $bytes;
 }
 
 ###############################################################################
@@ -391,6 +411,27 @@ sub t2ht ($) {
 
 ###############################################################################
 
+=item t2hj ($)
+
+Escapes text to look the same in JavaScript.
+
+ ' ->> \'
+ " ->> \"
+ \ ->> \\
+
+=cut
+
+sub t2hj ($) {
+    my $text=shift;
+    $text=~s/\\/\\\\/sg;
+    $text=~s/'/\\'/sg;
+    $text=~s/"/\\"/sg;
+    $text=~s/([\x00-\x1f])/'\\'.sprintf('%03o',ord($1))/esg;
+    return $text;
+}
+
+###############################################################################
+
 =back
 
 =head2 ARGUMENTS HANDLING
